/*
 * Created on 28.03.2005
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package de.wfn.builduserlibs;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.DecimalFormat;
import java.util.Iterator;
import java.util.Vector;
import java.util.zip.ZipEntry;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.jdt.core.IClasspathEntry;
import org.eclipse.jdt.internal.ui.wizards.buildpaths.CPListElement;
import org.eclipse.jdt.internal.ui.wizards.buildpaths.CPUserLibraryElement;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;

import de.wfn.builduserlibs.packageanalyse.PackageCentricZipFile;
import de.wfn.builduserlibs.packageanalyse.PackageList;

/**
 * @author riro
 */
public class Archive implements UserLibTreeItem
{
	/**
	 * Logger for this class
	 */
	private static final Logger logger = Logger.getLogger(Archive.class);

	/**
	 * Is this jar file subject to processing? 
	 */
	private boolean m_activated = true;
	
	/**
	 * Jar file path inside the zip file.
	 */
	private ZipEntry m_zipEntry;
	
	/**
	 * User library to which this jar file belongs.
	 */
	private UserLibZipFile m_parent;

	private PackageCentricZipFile m_pczf;

	private URL m_docURL;

	private URL m_sourceURL;

	private float m_docMatchFactor;

	private float m_sourceMatchFactor;
	
	/**
	 * Create a new Archive from a jar within a zip archive.
	 * @param zipEntry of the jar file 
	 * @param zipFile with the jar file is stored
	 * @throws IOException 
	 */
	public Archive(UserLibZipFile parent, ZipEntry zipEntry) throws IOException
	{
		m_parent = parent;
		m_zipEntry = zipEntry;
		File newFile = null;
		try
		{
		    newFile = FileTools.extractToTempFile(parent, zipEntry);
		    m_pczf = new PackageCentricZipFile(newFile);
		}
		finally
		{
			if (newFile != null)
			{
			    newFile.delete();
			}
		}
	}
	
	/**
	 * Get the name of the jar file.
	 * @return jar file's name
	 */
	public String getArchiveName()
	{
		return m_zipEntry.getName();
	}
	
	/**
	 * Get the reference to the parent zip file.
	 * @return parent zip file.
	 */
	public UserLibZipFile getUserLibZipFile()
	{
		return m_parent;
	}
	
	/* (non-Javadoc)
	 * @see de.wfn.builduserlibs.UserLibTreeItem#addToTree(org.eclipse.swt.widgets.Tree)
	 */
	public void addToTree(Tree tree)
	{
		// Intentionally left empty.
	}
	
	/* (non-Javadoc)
	 * @see de.wfn.builduserlibs.UserLibTreeItem#addToTree(org.eclipse.swt.widgets.TreeItem)
	 */
	public void addToTree(TreeItem treeItem)
	{
		TreeItem subTreeItem = new TreeItem(treeItem, SWT.NULL);
		setTreeItemText(subTreeItem, getArchiveName(), isActivated());
		subTreeItem.setData(this);

		//addSubTreeToTree(m_pczf.getClassPackageNames(), UserLibZipFile.SUB_TYPE_CLASS_PACKAGE, subTreeItem);
		//addSubTreeToTree(m_pczf.getJavaPackagesNames(), UserLibZipFile.SUB_TYPE_SRC_PACKAGE, subTreeItem);
		//addSubTreeToTree(m_pczf.getDocPackageNames(), UserLibZipFile.SUB_TYPE_DOC_PACKAGE, subTreeItem);

		TreeItem subSubTreeItem = new TreeItem(subTreeItem, SWT.NULL);
		DecimalFormat formatter = new DecimalFormat("##.00 %");
		if (m_sourceURL != null)
			setTreeItemText(subSubTreeItem, "Source = " + m_sourceURL.toExternalForm() /*+ " (" +	formatter.format(m_sourceMatchFactor) +")" */, isActivated());
		
		subSubTreeItem = new TreeItem(subTreeItem, SWT.NULL);
		if (m_docURL != null)
			setTreeItemText(subSubTreeItem, "Doc = " + m_docURL.toExternalForm() /* + " (" + formatter.format(m_docMatchFactor) + ")" */, isActivated());
	}
	
	/* (non-Javadoc)
	 * @see de.wfn.builduserlibs.UserLibTreeItem#toggleActivation()
	 */
	public boolean toggleActivation()
	{
		return m_activated = !m_activated ;
	}
	
	/* (non-Javadoc)
	 * @see de.wfn.builduserlibs.UserLibTreeItem#isActivated()
	 */
	public boolean isActivated()
	{
		return m_activated && getUserLibZipFile().isActivated();
	}
	
	/**
	 * Add a vector of items to a new tree node. Take care of the 
	 * activation flag of this Archive. 
	 * @param itemVector
	 * @param nodeName
	 * @param currentTreeItem
	 */
	private void addSubTreeToTree(Vector itemVector, String nodeName, TreeItem currentTreeItem)
	{
		if (itemVector != null && !itemVector.isEmpty())
		{
			TreeItem catTreeItem = new TreeItem(currentTreeItem, SWT.NULL);
			setTreeItemText(catTreeItem, nodeName, isActivated());
			Iterator it = itemVector.iterator();
			while (it.hasNext())
			{
				TreeItem subTreeItem = new TreeItem(catTreeItem, SWT.NULL);
				setTreeItemText(subTreeItem, (String)it.next(), isActivated());
			}
		}
	}

	/**
	 * Set the name of a tree item in dependency of the activation flag.
	 * @param treeItem
	 * @param text
	 * @param activation
	 */
	public static void setTreeItemText(TreeItem treeItem, String text, boolean activation)
	{
		if (!activation)
		{
			treeItem.setGrayed(true);
			treeItem.setText("[" + text + "]");
		}
		else
		{
			treeItem.setText(text);
		}
	}

	public PackageList getClassPackageList()
	{
		return m_pczf.getClassPackageList();
	}

	public PackageList getSourcePackageList()
	{
		return m_pczf.getJavaPackageList();
	}

	public Vector getDocPackageList()
	{
		Vector vec = new Vector();
		Iterator it = m_pczf.getDocPackageNames().iterator();
		while (it.hasNext())
		{
			vec.add(m_pczf.getDocPackageList((String)it.next()));
		}
		return vec;
	}

	public void setDocURL(URL containerURL)
	{
		m_docURL = containerURL;
	}

	public void setSourceURL(URL containerURL)
	{
		m_sourceURL = containerURL;
	}

	public void setDocMatchFactor(float maxMatchFactor)
	{
		m_docMatchFactor = maxMatchFactor;
	}

	public void setSourceMatchFactor(float maxMatchFactor)
	{
		m_sourceMatchFactor = maxMatchFactor;
	}

	public void extractTo(File target) throws IOException
	{
		FileTools.extractToDir(m_parent, m_zipEntry, target);
	}

	public void registerTo(CPUserLibraryElement newLibrary, File target)
	{
		/*
		 * Jar entry
		 */
		String path = new File(target.getAbsolutePath() + File.separatorChar + FileTools.filenameOfPath(m_zipEntry.getName())).getAbsolutePath();
		CPListElement newArchive = new CPListElement(newLibrary, null, IClasspathEntry.CPE_LIBRARY, Path.fromOSString(path), null);
		newLibrary.add(newArchive);
		
		if (m_sourceURL != null)		// Always a file 
		{
			path = target.getAbsolutePath() + File.separatorChar + FileTools.filenameOfPath(m_sourceURL.getFile());
			IPath sourceAttach= Path.fromOSString(path);
			newArchive.setAttribute(CPListElement.SOURCEATTACHMENT, sourceAttach);
		}
		if (m_docURL != null) 
		{
			try
			{
				String docDath = m_docURL.getPath().substring(m_docURL.getPath().indexOf('!'));
				String file = FileTools.filenameOfPath(m_docURL.getPath().substring(0,m_docURL.getPath().indexOf('!')));
				URL url = new URL("jar:file://" + target + "/" + file + docDath);
				newArchive.setAttribute(CPListElement.JAVADOC, url);
			}
			catch(MalformedURLException e)
			{
				// ignored.
			}
		}
	}

}
