/*
 * Created on 21.03.2005
 */
package de.wfn.builduserlibs;

import java.io.File;
import java.io.FilenameFilter;
import java.util.Iterator;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.IProgressMonitor;

/**
 * Scan directory hierarchies for certain file patterns.
 * @author riro
 */
public class DirectoryScanner
{
	/**
	 * Logger for this class
	 */
	private static final Logger logger = Logger.getLogger(DirectoryScanner.class);

	/**
	 * Vector of found File entries.
	 */
	private Vector m_vec;
	
	/**
	 * Extention to filter for.
	 */
	private String m_extension;
	
	/**
	 * Progress monitor to report current director to.
	 */
	private IProgressMonitor m_monitor;
	
	/**
	 * Create a directory scanner for a particular filename extension.
	 * @param extension 
	 */
	public DirectoryScanner(String extension, IProgressMonitor monitor)
	{
		logger.debug("--> DirectoryScanner(String extension)");
		m_vec = new Vector();
		m_extension = extension;
		m_monitor = monitor;
	}

	/**
	 * Remove previous found entries.
	 */
	public void reset()
	{
		m_vec.removeAllElements();
	}

	/**
	 * Get an iterator for the found files.
	 * @return Iterator
	 */
	public Iterator iterator()
	{
		return m_vec.iterator();
	}

	/**
	 * Scan for all zip files in a directory hierarchy.
	 * @param rootDir from which to scan for zip files.
	 * @param monitor 
	 * @return vector of Files containing zip files entries.
	 * @author jopi
	 * @since 05-04-29
	 */
	public void processDir(File rootDir)
	{
		scanDir(rootDir, new FilenameFilter()
		{
			public boolean accept(File dir, String name)
			{
				if (name.toLowerCase().endsWith(m_extension)) return true;
				StringBuffer speculativeDir = new StringBuffer(dir.getAbsolutePath());
				speculativeDir.append(File.separatorChar);
				speculativeDir.append(name);
				File myFile = new File(speculativeDir.toString());
				return myFile.isDirectory();
			}
		});
	}

	/**
	 * Check is File is a directory. 
	 * @param directory
	 * @param filenameFilter
	 */
	private void scanDir(File directory, FilenameFilter filenameFilter)
	{
		if (!directory.isDirectory())
		{
			return;
		}
		scan(directory, filenameFilter);
	}

	/**
	 * Recursivly scan all directories.
	 * @param directory
	 * @param filenameFilter
	 */
	private void scan(File directory, FilenameFilter filenameFilter)
	{
		File zipFiles[] = directory.listFiles(filenameFilter);
		File file;
		int len = zipFiles.length;
		for (int i = 0; i < len; i++)
		{
			file = zipFiles[i];
			if (file.isDirectory())
			{
				if (m_monitor != null)
					m_monitor.subTask("Scan directory : " + file.getAbsolutePath());
				scan(file, filenameFilter);
			}
			else
			{
				logger.debug("added " + file.getAbsolutePath());
				m_vec.addElement(file);
			}
		}
	}

	/**
	 * Number of files found during last scan process.
	 * @return number of files found.
	 */
	public int numberOfFiles()
	{
		return m_vec == null ? 0 : m_vec.size();
	}
}
