/*
 * Created on 28.03.2005
 */
package de.wfn.builduserlibs;

import java.io.File;
import java.lang.reflect.InvocationTargetException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.apache.log4j.Logger;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.jdt.core.ClasspathContainerInitializer;
import org.eclipse.jdt.core.IClasspathContainer;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.internal.ui.util.ExceptionHandler;
import org.eclipse.jdt.internal.ui.wizards.buildpaths.CPUserLibraryElement;
import org.eclipse.jdt.ui.JavaUI;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.ui.PlatformUI;

/**
 * The model which manage the data.
 * @author riro
 */
public class Model
{
	/**
	 * Logger for this class
	 */
	private static final Logger logger = Logger.getLogger(Model.class);

	/**
	 * Map between userlib name and the UserLib object.
	 */
	private HashMap m_userLibs;

	/**
	 * Root directory of the zip files to be analysed
	 */
	private Text m_zipRoot;

	/**
	 * Root directory underneath
	 */
	private Text m_targetRoot;

	/**
	 * Is location page completed?
	 */
	private boolean m_locationPageCompleted;

	/**
	 * Internal list used to be referenced from internal class
	 */
	private LinkedList m_list;
	
	/**
	 * Construct empty Model.
	 */
	public Model()
	{
		m_userLibs = new HashMap();
	}

	/**
	 * Iterator over all user libraries.
	 * @return
	 */
	private Iterator getUserLibIterator()
	{
		return m_userLibs.values().iterator();
	}

	/**
	 * @return Text composite of the zip root directory.
	 */
	public Text getZipRoot()
	{
		return m_zipRoot;
	}

	/**
	 * @param zipRoot Text composite to be set. 
	 */
	public void setZipRoot(Text zipRoot)
	{
		m_zipRoot = zipRoot;
	}

	/**
	 * @param dir to be set for zip root directory
	 */
	public void setZipRoot(String dir)
	{
		m_zipRoot.setText(dir);
		checkDirectoryExistence();
	}

	/**
	 * @return Text composite of the target root directory.
	 */
	public Text getTargetRoot()
	{
		return m_targetRoot;
	}

	/**
	 * @param targetRoot Text composite to be set.
	 */
	public void setTargetRoot(Text targetRoot)
	{
		m_targetRoot = targetRoot;
	}

	/**
	 * @param dir to be set for target root directory
	 */
	public void setTargetRoot(String dir)
	{
		m_targetRoot.setText(dir);
		checkDirectoryExistence();
	}

	/**
	 * @return Returns the locationPageCompleted.
	 */
	public boolean isLocationPageCompleted()
	{
		return m_locationPageCompleted;
	}

	/** 
	 * Remove a zip file from contributing to a user library.
	 * @param element
	 */
	public void removeZipFromUserLib(UserLibZipFile element)
	{
		UserLib userLib = element.getUserLib();
		userLib.removeZipFile(element);
	}

	public void addZipToUserLib(UserLib target, UserLibZipFile element)
	{
		target.addZipFile(element);
	}

	public boolean toggleActivation(TreeItem item)
	{
		if (item.getData() instanceof UserLib)
		{
			return ((UserLib)item.getData()).toggleActivation();
		}
		else
		{
			if (item.getData() instanceof Archive)
			{
				return ((Archive)item.getData()).toggleActivation();
			}
			else
			{
				if (item.getData() instanceof UserLibZipFile)
				{
					return ((UserLibZipFile)item.getData()).toggleActivation();
				}
			}
		}
		return true;
	}
	
	/**
	 * Check if target root and zip root directories exist and 
	 * reflect the result in the locationPageCompleted flag.
	 */
	public void checkDirectoryExistence()
	{
		m_locationPageCompleted = false;

		if (m_targetRoot == null || m_zipRoot == null)
		{
			return;
		}
		String data =  m_targetRoot.getText();  
		if (data != null && !"".equals(data) && new File(data).exists())
		{
			data = m_zipRoot.getText();
			if (data != null && !"".equals(data) && new File(data).exists())
			{
				m_locationPageCompleted = true;
			}
		}
	}

	/**
	 * Scan for the zip files under the zip root directory and populate the 
	 * selection tree.
	 * @param tree
	 */
	public void scanZipFiles(Tree tree)
	{
		InitialUserLibScanner op = new InitialUserLibScanner(new File(m_zipRoot.getText()));
		try
		{
			new ProgressMonitorDialog(tree.getShell()).run(true,true, op);
		}
		catch(InvocationTargetException e)
		{
			e.printStackTrace();
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}
		m_userLibs = op.getUserLibs();
		Iterator it = getUserLibIterator();
		while (it.hasNext())
		{
			((UserLib) it.next()).addToTree(tree);
		}
	}

	public boolean canFinish()
	{
		return isLocationPageCompleted() && m_userLibs.size() != 0;
	}

	public void extractJarFiles()
	{
		File target = new File(m_targetRoot.getText());
		Iterator it = getUserLibIterator();
		while (it.hasNext())
		{
			((UserLib) it.next()).extractJarTo(target);
		}
	}

	public void copyZipFiles()
	{
		File target = new File(m_targetRoot.getText());
		Iterator it = getUserLibIterator();
		while (it.hasNext())
		{
			((UserLib) it.next()).copyTo(target);
		}
	}

	public void registerUserLibs()
	{
		m_list = new LinkedList();
		File target = new File(m_targetRoot.getText());
		Iterator it = getUserLibIterator();
		while (it.hasNext())
		{
			Object obj = ((UserLib) it.next()).registerLib(target);
			if (obj != null)
			{
				m_list.add(obj);
			}
		}
		try {
			PlatformUI.getWorkbench().getProgressService().run(true, true, new IRunnableWithProgress() { 
				public void run(IProgressMonitor monitor) throws InvocationTargetException {
					try {
						if (monitor != null) {
							monitor= new NullProgressMonitor();
						}
						
						updateUserLibraries(m_list, monitor);
					} catch (CoreException e) {
						throw new InvocationTargetException(e);
					} finally {
						monitor.done();
					}
				}
			});
		} catch (InterruptedException e) {
			// cancelled by user
		} catch (InvocationTargetException e) {
			String title= "Registering user libraries";
			String message= "Registration failed.";
			ExceptionHandler.handle(e, m_targetRoot.getParent().getShell(), title, message);
		}
	}
	
	private void updateUserLibraries(List list, IProgressMonitor monitor) throws CoreException
	{
		HashSet oldNames = new HashSet(Arrays.asList(JavaCore.getUserLibraryNames()));
		int nExisting = list.size();

		HashSet newEntries = new HashSet(list.size());
		for (int i = 0; i < nExisting; i++)
		{
			CPUserLibraryElement element = (CPUserLibraryElement) list.get(i);
			boolean contained = oldNames.remove(element.getName());
			if (!contained)
			{
				newEntries.add(element);
			}
		}

		int len = nExisting + oldNames.size();
		monitor.beginTask("Registering userlibs", len); 
		MultiStatus multiStatus = new MultiStatus(JavaUI.ID_PLUGIN, IStatus.OK, "Error while registering userlib", null);

		ClasspathContainerInitializer initializer = JavaCore
				.getClasspathContainerInitializer(JavaCore.USER_LIBRARY_CONTAINER_ID);
		IJavaProject jproject = getPlaceholderProject();

		IJavaProject[] projectInit = new IJavaProject[] { jproject };
		IClasspathContainer[] containerInit = new IClasspathContainer[] { null };

		ArrayList paths = new ArrayList();
		ArrayList urls = new ArrayList();

		for (int i = 0; i < nExisting; i++)
		{
			CPUserLibraryElement element = (CPUserLibraryElement) list.get(i);
			IPath path = element.getPath();
			if (newEntries.contains(element) || element.hasChanges(JavaCore.getClasspathContainer(path, jproject)))
			{
				IClasspathContainer updatedContainer = element.getUpdatedContainer();
				try
				{
					initializer.requestClasspathContainerUpdate(path, jproject, updatedContainer);
					containerInit[0] = updatedContainer;
					JavaCore.setClasspathContainer(path, projectInit, containerInit, null); // force
																							// updating
																							// of
																							// containers,
																							// bug
																							// 62250
				}
				catch (CoreException e)
				{
					multiStatus.add(e.getStatus());
				}
			}
			element.collectJavaDocLocations(paths, urls);
			monitor.worked(1);
		}

		containerInit[0] = null;
		Iterator iter = oldNames.iterator();
		while (iter.hasNext())
		{
			String name = (String) iter.next();

			IPath path = new Path(JavaCore.USER_LIBRARY_CONTAINER_ID).append(name);
			try
			{
				initializer.requestClasspathContainerUpdate(path, jproject, null);
				JavaCore.setClasspathContainer(path, projectInit, containerInit, null); // force
																						// updating
																						// of
																						// containers,
																						// bug
																						// 62250
			}
			catch (CoreException e)
			{
				multiStatus.add(e.getStatus());
			}
			monitor.worked(1);
		}
		// save javadoc locations
		JavaUI.setLibraryJavadocLocations((IPath[]) paths.toArray(new IPath[paths.size()]), (URL[]) urls
				.toArray(new URL[paths.size()]));

		if (!multiStatus.isOK())
		{
			throw new CoreException(multiStatus);
		}
	}
	
	public static IJavaProject getPlaceholderProject()
	{
		String name = "####internal"; //$NON-NLS-1$
		IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
		while (true)
		{
			IProject project = root.getProject(name);
			if (!project.exists())
			{
				return JavaCore.create(project);
			}
			name += '1';
		}
	}

	public void updateLibEntries(Tree tree)
	{
		Iterator it = getUserLibIterator();
		TreeItem controls[] = tree.getItems();
		while (it.hasNext())
		{
			UserLib ul = (UserLib) it.next();
			if (ul.needsTreeBranchRefresh())
			{
				for (int i = 0; i < controls.length; i++)
				{
					if (ul == controls[i].getData())
					{
						ul.refreshTree(controls[i]);
						break;
					}
				}
			}
		}
	}

}
