/*
 * Created on 28.03.2005
 */
package de.wfn.builduserlibs;

import org.apache.log4j.Logger;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.IImportWizard;
import org.eclipse.ui.IWorkbench;

import de.wfn.builduserlibs.page.ZipFileRootLocationPage;
import de.wfn.builduserlibs.page.ZipFileSelectionPage;

/**
 * Import jar libraries stored in zip files and try to attach source and 
 * documentation files if available  
 * @author Jens-Oliver Siepmann
 */
public class UserLibImportWizard extends Wizard implements IImportWizard
{
	/**
	 * Logger for this class
	 */
	private static final Logger log = Logger.getLogger(UserLibImportWizard.class);

	/**
	 * Model for operations.
	 */
	private Model m_userLibModel;

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchWizard#init(org.eclipse.ui.IWorkbench, org.eclipse.jface.viewers.IStructuredSelection)
	 */
	public void init(IWorkbench workbench, IStructuredSelection selection)
	{
		log.debug("--> init()");
		m_userLibModel = new Model(); 
		setWindowTitle("Import user libraries");
		log.debug("<-- init()");
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.IWizard#performFinish()
	 */
	public boolean performFinish()
	{
		log.debug("--> performFinish()");
		m_userLibModel.copyZipFiles();
		m_userLibModel.extractJarFiles();
		m_userLibModel.registerUserLibs();
		log.debug("<-- performFinish()");
		return true;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.IWizard#canFinish()
	 */
	public boolean canFinish()
	{
		return m_userLibModel.canFinish();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.IWizard#addPages()
	 */
	public void addPages()
	{
		ZipFileRootLocationPage zipFileRootLocationPage = new ZipFileRootLocationPage("Location selection", m_userLibModel);
		zipFileRootLocationPage.setTitle("Location selection");
		zipFileRootLocationPage.setDescription("Please select the location of the zip files from where user libraries are extracted and define the target location where the resulting user library files are placed.");
		addPage(zipFileRootLocationPage);

		ZipFileSelectionPage zipFileSelectionPage = new ZipFileSelectionPage("File Selection", m_userLibModel);
		zipFileSelectionPage.setTitle("File selection/association");
		zipFileSelectionPage.setDescription("Please use drag and drop and the context menu to rearranged and deselect user libraries and zip files.");
		addPage(zipFileSelectionPage);
	}
}
