/*
 * Created on 09.05.2005 
 */
package de.wfn.builduserlibs;

import java.net.URL;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.jdt.core.ClasspathContainerInitializer;
import org.eclipse.jdt.core.IClasspathContainer;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.internal.ui.preferences.PreferencesMessages;
import org.eclipse.jdt.internal.ui.wizards.buildpaths.CPUserLibraryElement;
import org.eclipse.jdt.ui.JavaUI;

/**
 * Update the Eclipse user library repository 
 * @author Jens-Oliver Siepmann
 *
 */
public class UserLibraryUpdater
{

	IProgressMonitor m_progressMonitor;

	public UserLibraryUpdater(IProgressMonitor monitor)
	{
		m_progressMonitor = monitor;
	}

	private static IJavaProject getPlaceholderProject()
	{
		String name = "####internal"; //$NON-NLS-1$
		IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
		while (true)
		{
			IProject project = root.getProject(name);
			if (!project.exists())
			{
				return JavaCore.create(project);
			}
			name += '1';
		}
	}

	
	
	
	
	
	
	private void updateUserLibararies(List list) throws CoreException
	{
		/*
		 * Get the names of the currently registered user libraries.
		 */
		HashSet oldNames = new HashSet(Arrays.asList(JavaCore.getUserLibraryNames()));
		int nExisting = list.size();

		/*
		 * Create a list of user libraries which a currently not registered
		 * and remove all user libraries from the list of current user libaries
		 * which are subject to an update. 
		 */
		HashSet newEntries = new HashSet(list.size());
		for (int i = 0; i < nExisting; i++)
		{
			CPUserLibraryElement element = (CPUserLibraryElement) list.get(i);
			boolean contained = oldNames.remove(element.getName());
			if (!contained)
			{
				newEntries.add(element);
			}
		}

		/*
		 * Prepare progress indicator if needed. 
		 */
		MultiStatus multiStatus = new MultiStatus(JavaUI.ID_PLUGIN, 
				IStatus.OK, 
				PreferencesMessages.getString("UserLibraryPreferencePage.operation.error"), //$NON-NLS-1$ 
				null);

		if (m_progressMonitor != null)
		{
			int len = nExisting + oldNames.size();
			m_progressMonitor.beginTask(PreferencesMessages.getString("UserLibraryPreferencePage.operation"), //$NON-NLS-1$ 
					len);
		}

		/*
		 * Create dummy project its ClasspathContainer.
		 */
		ClasspathContainerInitializer initializer;
		IJavaProject jproject;
		initializer = JavaCore.getClasspathContainerInitializer(JavaCore.USER_LIBRARY_CONTAINER_ID);
		jproject = getPlaceholderProject();

		IJavaProject[] projectInit = new IJavaProject[] { jproject };
		IClasspathContainer[] containerInit = new IClasspathContainer[] { null };

		ArrayList paths = new ArrayList();
		ArrayList urls = new ArrayList();

		for (int i = 0; i < nExisting; i++)
		{
			CPUserLibraryElement element = (CPUserLibraryElement) list.get(i);
			IPath path = element.getPath();
			if (newEntries.contains(element) || element.hasChanges(JavaCore.getClasspathContainer(path, jproject)))
			{
				IClasspathContainer updatedContainer = element.getUpdatedContainer();
				try
				{
					initializer.requestClasspathContainerUpdate(path, jproject, updatedContainer);
					containerInit[0] = updatedContainer;
					JavaCore.setClasspathContainer(path, projectInit, containerInit, null); // force updating of containers, bug 62250 
				}
				catch (CoreException e)
				{
					multiStatus.add(e.getStatus());
				}
			}
			element.collectJavaDocLocations(paths, urls);
			if (m_progressMonitor != null)
				m_progressMonitor.worked(1);
		}

		containerInit[0] = null;
		Iterator iter = oldNames.iterator();
		while (iter.hasNext())
		{
			String name = (String) iter.next();

			IPath path = new Path(JavaCore.USER_LIBRARY_CONTAINER_ID).append(name);
			try
			{
				initializer.requestClasspathContainerUpdate(path, jproject, null);
				JavaCore.setClasspathContainer(path, projectInit, containerInit, null); // force updating of containers, bug 62250 
			}
			catch (CoreException e)
			{
				multiStatus.add(e.getStatus());
			}
			if (m_progressMonitor != null)
				m_progressMonitor.worked(1);
		}
		// save javadoc locations
		JavaUI.setLibraryJavadocLocations((IPath[]) paths.toArray(new IPath[paths.size()]), 
				(URL[]) urls.toArray(new URL[paths.size()]));

		if (!multiStatus.isOK())
		{
			throw new CoreException(multiStatus);
		}
	}

}
