/*
 * Created on 23.04.2005
 */
package de.wfn.builduserlibs.inspectors;

import java.io.File;
import java.io.IOException;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import org.apache.log4j.Logger;

import de.wfn.builduserlibs.ArchiveExtractor;
import de.wfn.builduserlibs.FileTools;

/**
 * Scan a zip archive for jar files and collects information. 
 * @author Jens-Oliver Siepmann
 */
public class JarInspector
{
	/**
	 * Logger for this class
	 */
	private static final Logger logger = Logger.getLogger(JarInspector.class);

	/**
	 * Map between jar filenames and their zip entries.
	 */
	private HashMap m_jar2ZipEntryMap;
	
	/**
	 * Zip file which was analyzed
	 */
	private ZipFile m_zipFile;
	
	/**
	 * Default Constructor, does nothing.
	 */
	public JarInspector()
	{
		/* Intentionally left empty */
	}

	/**
	 * Analyse zip file and scan for jar files.
	 * @param zipFile
	 */
	public void scan(ZipFile zipFile)
	{
		m_zipFile = zipFile;
		m_jar2ZipEntryMap = new HashMap();
		
		Enumeration zipEnum = zipFile.entries();
		while (zipEnum.hasMoreElements())
		{
			ZipEntry zipEntry = (ZipEntry) zipEnum.nextElement();
			if (!zipEntry.isDirectory() && zipEntry.getName().toLowerCase().endsWith(".jar"))
			{
				m_jar2ZipEntryMap.put(FileTools.filenameOfPath(zipEntry.getName()),zipEntry);
			}
		}
	}
	
	public void extract(File rootDir) throws IOException
	{
		rootDir.mkdirs();
		
		Iterator jarIt = m_jar2ZipEntryMap.values().iterator();
		ArchiveExtractor ae = new ArchiveExtractor(rootDir.getAbsolutePath());
		while (jarIt.hasNext())
		{
			ae.extractZipEntryToDestination(rootDir, m_zipFile, (ZipEntry)jarIt.next(), false);
		}
	}
	
	/**
	 * @return Returns the jar2ZipEntry.
	 */
	public HashMap getJar2ZipEntryMap()
	{
		return m_jar2ZipEntryMap;
	}
	
	/**
	 * @return Returns iterator for jar filename.
	 */
	public Iterator getJarNameIterator()
	{
		return m_jar2ZipEntryMap.keySet().iterator();
	}
	
	/**
	 * @return Returns the zipFile.
	 */
	public ZipFile getZipFile()
	{
		return m_zipFile;
	}
}
