/*
 * Created on 10.05.2005 
 */
package de.wfn.builduserlibs.packageanalyse;

import java.io.File;
import java.net.URL;
import java.util.Arrays;
import java.util.Iterator;
import java.util.TreeSet;
import java.util.Vector;

/**
 * Manages a list of package.
 * @author Jens-Oliver Siepmann
 *
 */
public class PackageList
{
	protected URL m_container;
	
	private TreeSet m_set;
	
	public PackageList()
	{
		m_set = new TreeSet();
	}
	
	public URL getContainerURL()
	{
		return m_container;
	}

	public void setContainerURL(URL url)
	{
		m_container = url;
	}

	public Vector toVector()
	{
		return new Vector(Arrays.asList(m_set.toArray()));
	}
	
	public void addPackage(String packageName)
	{
		m_set.add(toPackageNotation(packageName));
	}
	
	public void addPackageWithFile(String packageName)
	{
		m_set.add(stripFilenamePart(toPackageNotation(packageName)));
	}
	
	public void addPackage(File packageName)
	{
		String pkg = toPackageNotation(packageName.getAbsolutePath());
		if (packageName.isFile())
		{
			pkg = stripFilenamePart(pkg);
		}
		m_set.add(pkg);
	}
	
	public float compare(PackageList packageList)
	{
		Iterator it = m_set.iterator();
		if (size() == 0) return 0;
		int inCounter = 0;
		while (it.hasNext())
		{
			if (packageList.m_set.contains(it.next()))
				inCounter++;
		}
		return inCounter/size();
	}
	
	public int size()
	{
		return m_set.size();
	}
	
	private String stripFilenamePart(String path)
	{
		int lastDot = path.lastIndexOf('.');
		if (lastDot == -1) return "";
		return path.substring(0, lastDot);
	}

	private String toPackageNotation(String path)
	{
		String ret = path;
		if (path.indexOf('/') != -1) ret = ret.replace('/', '.'); 
		if (path.indexOf('\\') != -1) ret = ret.replace('\\', '.'); 
		return ret;
	}
	
	
}
