/*
 * Created on 10.04.2005
 */
package de.wfn.builduserlibs.page;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Text;

import de.wfn.builduserlibs.Model;


/**
 * On this page the user specifies the directory under which the zip files are
 * found. Additionally a target directory is specified into which the zip files
 * are extracted on a user library basis.
 * 
 * @author Jens-Oliver Siepmann
 */
public class ZipFileRootLocationPage extends WizardPage
{
	/**
	 * Structural member variable to enable the temporary interface
	 * implementation to access outside data.
	 */
	private ZipFileRootLocationPage m_thisPage;

	/**
	 * Reference the model for storing source and target path on 
	 * "next page" event.
	 */
	private Model m_model;

	/**
	 * Construct the page with a page name.
	 */
	public ZipFileRootLocationPage(String pageName, Model model)
	{
		super(pageName);
		m_thisPage = this;
		m_model = model;
	}

	/**
	 * Setup the page. Create all controls and their listeners.
	 */
	public void createControl(Composite parent)
	{
		Composite composite = new Composite(parent, SWT.NONE);

		setPageComplete(false);
		
		GridLayout pageGridLayout = new GridLayout();
		pageGridLayout.numColumns = 2;
		composite.setLayout(pageGridLayout);

		Group group0 = new Group (composite, SWT.NONE);
		group0.setText ("Location of ZIP files");
		GridData data = new GridData ();
		data.widthHint = 600;
		data.horizontalSpan = 2;
		data.verticalSpan = 1;
		data.grabExcessHorizontalSpace = true;
		group0.setLayoutData (data);

		GridLayout group0GridLayout = new GridLayout();
		pageGridLayout.numColumns = 2;
		group0.setLayout(pageGridLayout);
		
		createZipRootDirText(group0);
		createZipRootDirBrowserBtn(group0);

		group0 = new Group (composite, SWT.NONE);
		group0.setText ("Location of target directory for user libraries");
		data = new GridData ();
		data.widthHint = 600;
		data.horizontalSpan = 2;
		data.verticalSpan = 1;
		data.grabExcessHorizontalSpace = true;
		group0.setLayoutData (data);

		group0GridLayout = new GridLayout();
		pageGridLayout.numColumns = 2;
		group0.setLayout(pageGridLayout);
		
		createTargetRootDirText(group0);
		createTargetRootDirBrowserBtn(group0);
		
		
		setControl(composite);
	}
	
	/**
	 * Create control for input of zip file's root directory.
	 * @param composite
	 * @return
	 */
	private void createTargetRootDirText(Composite composite)
	{
		Text text = new Text(composite, SWT.SINGLE | SWT.BORDER);
		text.setText(System.getProperties().getProperty("TARGETROOT", ""));
		m_model.setTargetRoot(text);
		GridData data = new GridData();
		data.widthHint = 500;
		data.grabExcessHorizontalSpace = true;
		text.setLayoutData(data);
		text.addKeyListener(new KeyListener()
		{
			public void keyPressed(KeyEvent e){}

			public void keyReleased(KeyEvent e)
			{
				m_model.checkDirectoryExistence();
				setPageComplete(m_model.isLocationPageCompleted());
			}
		});
	}
	
	/**
	 * Create control to browse for zip file's root directory.
	 * @param composite
	 */
	private void createTargetRootDirBrowserBtn(Composite composite)
	{
		Button button1 = new Button(composite, SWT.PUSH);
		button1.setText("browse ...");
		GridData data = new GridData();
		data.horizontalAlignment = GridData.END;
		button1.setLayoutData(data);
		button1.addSelectionListener(new SelectionListener()
		{
			public void widgetDefaultSelected(SelectionEvent e) {}

			public void widgetSelected(SelectionEvent e)
			{
				DirectoryDialog dialog = new DirectoryDialog(e.widget.getDisplay().getActiveShell());
				String dir = m_model.getTargetRoot().getText();
				dialog.setFilterPath(dir == null ? "c:\\" : dir);
				dir = dialog.open();
				if (dir != null)
				{
					m_model.setTargetRoot(dir);
					setPageComplete(m_model.isLocationPageCompleted());
				}
			}
		});
	}

	/**
	 * Create control for input of zip file's root directory.
	 * @param composite
	 * @return
	 */
	private void createZipRootDirText(Composite composite)
	{
		Text text = new Text(composite, SWT.SINGLE | SWT.BORDER);
		text.setText(System.getProperties().getProperty("ZIPROOT",""));
		m_model.setZipRoot(text);
		GridData data = new GridData();
		data.widthHint = 500;
		data.grabExcessHorizontalSpace = true;
		text.setLayoutData(data);
		text.addKeyListener(new KeyListener()
		{
			public void keyPressed(KeyEvent e) {}

			public void keyReleased(KeyEvent e)
			{
				m_model.checkDirectoryExistence();
				setPageComplete(m_model.isLocationPageCompleted());
			}
		});
	}

	/**
	 * Create control to browse for zip file's root directory.
	 * @param composite
	 */
	private void createZipRootDirBrowserBtn(Composite composite)
	{
		Button button1 = new Button(composite, SWT.PUSH);
		button1.setText("browse ...");
		GridData data = new GridData();
		data.horizontalAlignment = GridData.END;
		button1.setLayoutData(data);
		button1.addSelectionListener(new SelectionListener()
		{
			public void widgetDefaultSelected(SelectionEvent e) {}
			
			public void widgetSelected(SelectionEvent e)
			{
				DirectoryDialog dialog = new DirectoryDialog(e.widget.getDisplay().getActiveShell());
				String dir = m_model.getZipRoot().getText();
				dialog.setFilterPath(dir == null ? "c:\\" : dir);
				dir = dialog.open();
				if (dir != null)
				{
					m_model.setZipRoot(dir);
					setPageComplete(m_model.isLocationPageCompleted());
				}
			}
		});
	}
}
