/*
 * Created on 06.04.2005
 */
package de.wfn.builduserlibs.page;

import org.apache.log4j.Logger;
import org.eclipse.jface.action.Action;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DragSource;
import org.eclipse.swt.dnd.DragSourceAdapter;
import org.eclipse.swt.dnd.DragSourceEvent;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.DropTargetAdapter;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;

import de.wfn.builduserlibs.Model;
import de.wfn.builduserlibs.UserLib;
import de.wfn.builduserlibs.UserLibTreeItem;
import de.wfn.builduserlibs.UserLibZipFile;
import de.wfn.builduserlibs.UserLibZipFileTransfer;

/**
 * @author riro
 */
public class ZipFileSelectionPageView
{
	/**
	 * Logger for this class
	 */
	private static final Logger logger = Logger.getLogger(ZipFileSelectionPageView.class);

	protected Model m_model;
	protected TreeItem m_lastClickedItem;
	protected ZipFileSelectionPage m_page;
	
	public ZipFileSelectionPageView(ZipFileSelectionPage page, Model model)
	{
		m_model = model;
		m_page = page;
	}

	public Tree createTreeControl(Composite composite)
	{
	
		final Tree tree = new Tree(composite, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL | SWT.MULTI);
		
		//final TreeEditor editor = new TreeEditor(m_tree);

		configureDrag(tree);
		configureContextMenue(tree);
	
		tree.addListener (SWT.MouseDown, new Listener () {
			public void handleEvent (Event event) {
				Point point = new Point (event.x, event.y);
				TreeItem item = tree.getItem (point);
				if (item != null) {
					m_lastClickedItem = item;
				}
			}
		});
		
/*
		// Create the text field
		final Text text = new Text(composite, SWT.NONE);
	
		m_tree.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				if (logger.isDebugEnabled())
				{
					logger.debug("widgetSelected(SelectionEvent) - start");
				}
	
				// Clean up any previous editor control
				Control oldEditor = editor.getEditor();
				if (oldEditor != null) oldEditor.dispose();
	
				// Identify the selected row
				TreeItem item = (TreeItem) e.item;
	
				// The control that will be the editor must be a child of the
				// Tree
				Text text = new Text(m_tree, SWT.NONE);
	
				// The text editor must have the same size as the cell and must
				// not be any smaller than 50 pixels.
				editor.horizontalAlignment = SWT.LEFT;
				editor.grabHorizontal = true;
				editor.minimumWidth = 50;
	
				// Open the text editor on the selected row.
				editor.setEditor(text, item);
	
				// Assign focus to the text control
				text.setFocus();
	
				if (logger.isDebugEnabled())
				{
					logger.debug("widgetSelected(SelectionEvent) - end");
				}
			}
		});
		m_tree.addListener(SWT.MouseDown, new Listener()
		{
			public void handleEvent(Event event)
			{
				if (logger.isDebugEnabled())
				{
					logger.debug("handleEvent(Event) - start");
				}
	
				Point point = new Point(event.x, event.y);
				TreeItem item = m_tree.getItem(point);
				if (item != null)
				{
					System.out.println("Mouse down: " + item);
				}
	
				if (logger.isDebugEnabled())
				{
					logger.debug("handleEvent(Event) - end");
				}
			}
		});
		m_tree.addListener(SWT.Collapse, new Listener()
		{
			public void handleEvent(Event event)
			{
				if (logger.isDebugEnabled())
				{
					logger.debug("handleEvent(Event) - start");
				}
	
				TreeItem item = (TreeItem) event.item;
				if (item != null)
				{
					System.out.println("Collapsing : " + item);
				}
	
				if (logger.isDebugEnabled())
				{
					logger.debug("handleEvent(Event) - end");
				}
			}
		});
*/	
		tree.setSize(400, 200);
		return tree;
	}

	private Action deleteAction;
	
	private void configureContextMenue(final Tree tree)
	{
		final Menu menu = new Menu(tree.getShell(), SWT.POP_UP);
		tree.setMenu(menu);
		menu.addListener(SWT.Show, new Listener()
		{
			public void handleEvent(Event event)
			{
				logger.debug("Current clicked item = " + m_lastClickedItem);

				MenuItem[] menuItems = menu.getItems();
				for (int i = 0; i < menuItems.length; i++)
				{
					menuItems[i].dispose();
				}

				/*
				 * Decide how to configure the menu
				 */
				TreeItem [] treeItems = tree.getSelection();
				for (int i = 0; i < treeItems.length; i++)
				{
					if (!(treeItems[i].getData() instanceof UserLibTreeItem))
						return;
				}
				MenuItem menuItem = new MenuItem(menu, SWT.PUSH);
				menuItem.setText("Toggle activation");
				menuItem.addSelectionListener(new SelectionListener()
				{


					public void widgetSelected(SelectionEvent e)
					{
						TreeItem[] treeItems = tree.getSelection();
						for (int i = 0; i < treeItems.length; i++)
						{
							boolean flag = m_model.toggleActivation(treeItems[i]);
							String text = treeItems[i].getText();
							if (flag)
							{
								if (text != null && text.charAt(0) == '[' && text.charAt(text.length()-1) == ']')
								{
									text = text.substring(1, text.length()-1);
								}
								treeItems[i].setGrayed(false);
							}
							else
							{
								text = "[" + text + "]";
								treeItems[i].setGrayed(true);
							}
							treeItems[i].setText(text);
						}
					}

					public void widgetDefaultSelected(SelectionEvent e)
					{
						logger.debug("widgetDefaultSelected(Event)");
					}
				});
			}
		});
	}
	
	/**
	 * @param m_tree
	 */
	private void configureDrag(final Tree tree)
	{
		DragSource ds = new DragSource(tree, DND.DROP_MOVE);
		ds.setTransfer(new Transfer[] { UserLibZipFileTransfer.getInstance() });
		ds.addDragListener(new DragSourceAdapter()
		{
			public void dragFinished(DragSourceEvent event)
			{
				if (event.detail != DND.DROP_NONE)
				{
					logger.info("dragFinished called.");
					TreeItem ti[] = tree.getSelection();
					for (int i = 0; i < ti.length; i++)
					{
						if (ti[i].getData() instanceof UserLibZipFile)
						{
							logger.info("Selected data = " + ((UserLibZipFile)ti[i].getData()).getFileNamePart());
							m_model.removeZipFromUserLib((UserLibZipFile)ti[i].getData());
							ti[i].dispose();
						}
					}
					m_model.updateLibEntries(tree);
				}
				else
				{
					logger.info("dragFinished called. - NO OPERATION");
				}
			}
			
			public void dragSetData(DragSourceEvent event)
			{
				logger.info("dragSetData called.");
				// Set the data to be the first selected item's text
				TreeItem ti[] = tree.getSelection();
				UserLibZipFile ulzf[] = new UserLibZipFile [ti.length]; 
				for (int i = 0; i < ti.length; i++)
				{
					ulzf[i] = (UserLibZipFile)(ti[i].getData());
				}
				event.data = ulzf;
			}
			
			public void dragStart(DragSourceEvent event)
			{
				logger.info("dragStart called.");
				if (!(tree.getSelection()[0].getData() instanceof UserLibZipFile))
				{
					event.doit = false;
				}
			}
		});
		
		// Create the drop target on the text field
		DropTarget dt = new DropTarget(tree, DND.DROP_MOVE);
		dt.setTransfer(new Transfer[] { UserLibZipFileTransfer.getInstance() });
		dt.addDropListener(new DropTargetAdapter()
		{
//			public void dragEnter(DropTargetEvent event)
//			{
//				super.dragEnter(event);
//				logger.info("dragEnter called. - NO OPERATION");
//			}
//
//			public void dragLeave(DropTargetEvent event)
//			{
//				super.dragLeave(event);
//				logger.info("dragLeave called. - NO OPERATION");
//			}
//			 
//			public void dragOperationChanged(DropTargetEvent event) 
//			{
//				super.dragOperationChanged(event);
//				logger.info("dragOperationChanged called. - NO OPERATION");
//			}

			public void dragOver(DropTargetEvent event) 
			{
				event.feedback = DND.FEEDBACK_SELECT | DND.FEEDBACK_SCROLL | DND.FEEDBACK_EXPAND;
				if (getTargetUserLib(event) == null)
				{
					event.detail = DND.DROP_NONE;
				}
				else
				{
					if (event.detail != DND.DROP_COPY || event.detail != DND.DROP_MOVE)
					{
						event.detail = DND.DROP_MOVE;
					}
				}
			}

			public void dropAccept(DropTargetEvent event) 
			{
				if (event.item instanceof TreeItem)
				{
					logger.info("Accepting Node = " + ((TreeItem)event.item).getData());
					if (((TreeItem)event.item).getData() instanceof UserLib)
					{
						logger.info("Accepting Userlib = " + ((UserLib)((TreeItem)event.item).getData()).getName());
					}
					else
					{
						event.detail = DND.DROP_NONE;
					}
				}
				else
				{
					event.detail = DND.DROP_NONE;
				}
			}

			
			public void drop(DropTargetEvent event)
			{
				if (event.detail == DND.DROP_COPY || event.detail == DND.DROP_MOVE)
				{
					logger.info("drop called.");
					if (UserLibZipFileTransfer.getInstance().isSupportedType(event.currentDataType))
					{
						UserLib userLib = ((UserLib)((TreeItem)event.item).getData()); 
						logger.info("Inserting to Userlib = " + userLib.getName());
						UserLibZipFile ulzf[] = (UserLibZipFile[])event.data;
						for (int i = 0; i < ulzf.length; i++)
						{
							logger.info("UserLibZipFile to insert = " + ulzf[i].getFileNamePart());
							m_model.addZipToUserLib(userLib, ulzf[i]);
							ulzf[i].addToTree((TreeItem)event.item);
						}
					}
				}
				m_model.updateLibEntries(tree);
			}
			
			private UserLib getTargetUserLib(DropTargetEvent event) 
			{
				// Determine the target File for the drop 
				TreeItem item = tree.getItem(tree.toControl(new Point(event.x, event.y)));
				UserLib target = null;
				if (item != null) {
					// We are over a particular item in the m_tree, use the item's file
					try
					{
						target = (UserLib) item.getData();
					}
					catch(ClassCastException e)
					{
					}
				}
				return target;
			}
		});
	}
}
